"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.formatTestCaseAttempt = void 0;
const indent_string_1 = __importDefault(require("indent-string"));
const status_1 = __importDefault(require("../../status"));
const figures_1 = __importDefault(require("figures"));
const location_helpers_1 = require("./location_helpers");
const test_case_attempt_parser_1 = require("./test_case_attempt_parser");
const step_argument_formatter_1 = require("./step_argument_formatter");
const value_checker_1 = require("../../value_checker");
const CHARACTERS = {
    [status_1.default.AMBIGUOUS]: figures_1.default.cross,
    [status_1.default.FAILED]: figures_1.default.cross,
    [status_1.default.PASSED]: figures_1.default.tick,
    [status_1.default.PENDING]: '?',
    [status_1.default.SKIPPED]: '-',
    [status_1.default.UNDEFINED]: '?',
};
function getStepMessage(testStep) {
    switch (testStep.result.status) {
        case status_1.default.AMBIGUOUS:
        case status_1.default.FAILED:
            return testStep.result.message;
        case status_1.default.UNDEFINED:
            return `${'Undefined. Implement with the following snippet:' + '\n\n'}${indent_string_1.default(testStep.snippet, 2)}\n`;
        case status_1.default.PENDING:
            return 'Pending';
    }
    return '';
}
function formatStep({ colorFns, testStep }) {
    const { result: { status }, actionLocation, attachments, } = testStep;
    const colorFn = colorFns.forStatus(status);
    const identifier = testStep.keyword + value_checker_1.valueOrDefault(testStep.text, '');
    let text = colorFn(`${CHARACTERS[status]} ${identifier}`);
    if (value_checker_1.doesHaveValue(actionLocation)) {
        text += ` # ${colorFns.location(location_helpers_1.formatLocation(actionLocation))}`;
    }
    text += '\n';
    if (value_checker_1.doesHaveValue(testStep.argument)) {
        const argumentsText = step_argument_formatter_1.formatStepArgument(testStep.argument);
        text += indent_string_1.default(`${colorFn(argumentsText)}\n`, 4);
    }
    attachments.forEach(({ body, mediaType }) => {
        const message = mediaType === 'text/plain' ? `: ${body}` : '';
        text += indent_string_1.default(`Attachment (${mediaType})${message}\n`, 4);
    });
    const message = getStepMessage(testStep);
    if (message !== '') {
        text += `${indent_string_1.default(colorFn(message), 4)}\n`;
    }
    return text;
}
function formatTestCaseAttempt({ colorFns, cwd, snippetBuilder, supportCodeLibrary, testCaseAttempt, }) {
    const parsed = test_case_attempt_parser_1.parseTestCaseAttempt({
        cwd,
        snippetBuilder,
        testCaseAttempt,
        supportCodeLibrary,
    });
    let text = `Scenario: ${parsed.testCase.name}`;
    text += getAttemptText(parsed.testCase.attempt, parsed.testCase.worstTestStepResult.willBeRetried);
    text += ` # ${colorFns.location(location_helpers_1.formatLocation(parsed.testCase.sourceLocation))}\n`;
    parsed.testSteps.forEach((testStep) => {
        text += formatStep({ colorFns, testStep });
    });
    return `${text}\n`;
}
exports.formatTestCaseAttempt = formatTestCaseAttempt;
function getAttemptText(attempt, willBeRetried) {
    if (attempt > 0 || willBeRetried) {
        const numberStr = (attempt + 1).toString();
        const retriedStr = willBeRetried ? ', retried' : '';
        return ` (attempt ${numberStr}${retriedStr})`;
    }
    return '';
}
//# sourceMappingURL=test_case_attempt_formatter.js.map